#ifndef DEFINITION_H_INCLUDED_
#define DEFINITION_H_INCLUDED_

#include <string>

#include "rose.h"


namespace risc {

class Class;

class Definition {

public:

  /**
   * \brief default constructor
   */
  explicit Definition(SgNode *ast_node);

  /**
   * \brief  Returns the name of the defintion
   */
  virtual std::string get_name();

  /**
   * \brief  Returns the associated ast node of the definition
   */
  virtual SgNode* get_ast_node();

  /**
   * \brief  Returns the type of the ast node
   */
  virtual SgType* get_ast_type();

  /**
   * \brief  Returns the name of the type of the ast node
   */
  virtual std::string get_ast_type_name();

  /**
   * \brief Determines if a ast node has an associated ast node
   */
  bool has_source_location();

  /**
   * \brief  Get the name of the file where the declaration is located
   * \return filename
   */
  std::string get_file_name();

  /**
   * \brief  Get the line number of the instance
   * \return line number
   */
  int get_line_number();

  /**
   * \brief  Get the position of the in declaration in the corresponding line
   *         In other words the column in the line
   * \return position in line
   */
  int get_position_in_line();

  SgNode* ast_node_;
  SgType* ast_type_pointer_;

private:
  Definition(const Definition& rhs);
};


} // end of namespace risc


#endif /* DEFINITION_H_INCLUDED_ */

/* ex: set softtabstop=2 tabstop=2 shiftwidth=2 expandtab: */
