#ifndef DESGIN_H_INCLUDED_
#define DESGIN_H_INCLUDED_

#include "rose.h"

#include "module.h"
#include "primitive_channel.h"
#include "hierarchical_channel.h"

#include "event.h"

#include "thread.h"
#include "cthread.h"
#include "method.h"
#include "function.h"

namespace risc {

/**
 * This class represents the internal representation. The IR offers different
 * interfaces to start search in IR.
 */
class Design {

public:

  /**
   * \brief default constructor
   * \param argc  expects argc from main function
   * \param argv  expects argv from main function
   */
  explicit Design(int argc, char* argv[]);
  ~Design();

  void initialize_defintions();
  void initialize_declarations();
  void initialize_top_modules();


  //////////////////////// Functions to print vectors ////////////////////////
  void print_all_class_definitions();
  void print_all_primitive_channel_definitions();
  void print_all_hierarchical_channel_definitions();
  void print_all_events();
  void print_all_threads();
  void print_all_cthreads();
  void print_all_methods();
  void print_all_functions();
  void print_design();


  /////////////////// Vectors of different design elements ///////////////////
  ModuleVector module_definitions_;
  ModuleInstanceVector top_modules_;

  PrimitiveChannelVector primitive_channel_;
  HierarchicalChannelVector hierarchical_channel_;

  EventVector events_;
  VariableVector variables_; // global variables

  ThreadVector threads_;
  CThreadVector cthreads_;
  MethodVector methods_;
  FunctionVector functions_;


  /////////////////////// Functions to search in design //////////////////////
  Module* find_module_definition(SgClassDefinition* cd);


  /////////////////////////// Rose specific elements /////////////////////////
  SgProject* project_;
  SgFunctionDeclaration* main_;


  ////////////////////////// SystemC specific elements ///////////////////////
  /**
   * \brief Pointer to the sc_module definition in the rose ast tree
   */
  SgClassDefinition* sc_module_;
  SgClassDefinition* sc_port_;
  SgClassDefinition* sc_port_base_;
  SgClassDefinition* sc_in_;
  SgClassDefinition* sc_out_;
  SgClassDefinition* sc_inout_;
  SgClassDefinition* sc_event_;
  SgClassDefinition* sc_channel_;
  SgClassDefinition* sc_prim_channel_;

  /**
   * \brief Pointer to the sc_main function declaration
   */
  SgFunctionDeclaration* sc_main_;

private:
  /**
   * \brief Disabled copy constructor
   */
  Design(const Design &d);
};

class GlobalDesign {
public:
  static Design *design_static;
};
Design* get_design();
SgProject* get_project();

}; // end namespace risc

#endif /* DESGIN_H_INCLUDED_ */

/* ex: set softtabstop=2 tabstop=2 shiftwidth=2 expandtab: */
