#ifndef FUNCTION_H_INCLUDED_
#define FUNCTION_H_INCLUDED_

#include <vector>
#include <string>

#include "rose.h"

#include "definition.h"

namespace risc {

class Function: public Definition {

public:

  /**
   * \brief Default constructor
   */
  explicit Function(SgFunctionDefinition *ast_node);

  /**
   * \brief  Returns the ast definition of the function
   * \return Pointer to the definition
   */
  SgFunctionDefinition* get_ast_node();

  /**
   * \brief  Returns the name of the type of the ast node
   */
  virtual std::string get_ast_type_name();

  /**
   * \brief  returns the name of the function definition
   */
  std::string get_name();

private:
  /**
   * \brief Disabled copy constructor
   */
  Function(const Function &f);
};

typedef std::vector<Function*> FunctionVector;
typedef std::vector<Function*>::iterator FunctionVectorIter;
typedef std::vector<Function*>::const_iterator FunctionVectorConstIter;

}; // end namespace risc

#endif /* FUNCTION_H_INCLUDED_ */

/* ex: set softtabstop=2 tabstop=2 shiftwidth=2 expandtab: */
