#ifndef MISC_H_INCLUDED_
#define MISC_H_INCLUDED_

#include "rose.h"
#include <string>

namespace risc
{

namespace tools
{

/**
 * \brief               This function checks if the passed type
 *                      represents a data structure
 * \param type          Type that will be checked
 * \return              True, if the type represents a data structure
 */
bool is_data_structure(SgType* type);

/**
 * \brief               This function checks if the passed type
 *                      represents a C++ builtin type
 * \param type          Type that will be checked
 * \return              True, if the type represents a C++ builtin type
 */
bool is_builtin_type(SgType* type);


/**
 * \brief               Compares two classes
 * \param cd1           Class definition 1
 * \param cd2           Class definition 2
 * \return              True, if the types are equal
 */
bool class_types_are_identical(SgClassDefinition* cd1, SgClassDefinition* cd2);

/**
 * \brief               Compares the function name with the given string
 * \param function      The function declaration
 * \param name          String which will be compared
 * \return              True, if the string and the function name are equal
 */
bool is_func_name(const SgFunctionDeclaration* function, const std::string& name);

/**
 * \brief               Prints a string with indention
 * \param s             Sting to print
 * \param indent        Indention level
 */
void print_indented(std::string s, unsigned indent);

/**
 * \brief               Determines the class defintion of varialbe
 *                      It is expected that the type is SgClassType
 * \param var_decl      The given variable declaration
 * \return              Associated SgClassDefintion of the var_decl
 */
SgClassDefinition* class_definition_of_variable(SgVariableDeclaration* var_decl);

}; // end namespace tools

}; // end namespace risc

#endif /* MISC_H_INCLUDED_ */

/* ex: set softtabstop=2 tabstop=2 shiftwidth=2 expandtab: */
